program TopCut_
  !! cut wavefield on topography
  !!
  !! topo = x + i z
  use sep
  use util_

  implicit none
  character(len=128) :: name

  real,    pointer :: wfld(:,:)
  real,    pointer :: data(:)
  complex, pointer :: topo(:)

  type(myaxis) :: ax,az,at,as
  integer      :: ix,iz,it,is
  real         ::  x, z,tt
  real         :: zo,fz
  real         :: xo,fx

  real         :: w00,w10,w01,w11

  call sep_init()
  call hinaxis(az,1)
  call hinaxis(ax,2)
  call hinaxis(at,3)

  name="topo"
  call ainaxis(as,1,name)

  call houaxis(as,1)
  call houaxis(at,2)
  call to_history("n3",1)

  call to_history("esize",4)
  call sep_close()

  allocate(wfld(az%n,ax%n))
  allocate(topo(as%n))
  allocate(data(as%n))

  write(0,*) "create topography mask"
  call sreed("topo",topo,size(topo)*8)

  write(0,*) "create data"
  do it=1,at%n
     write(0,*) it,at%n
     call sep_read(wfld)

     do is=1,as%n
        z =aimag(topo(is))
        x = real(topo(is))

        iz=floor(( z-az%o)/az%d+1)
        if(iz<   1+1) cycle
        if(iz>az%n-1) cycle
        zo = az%o + (iz-1)*az%d
        fz = ( z-zo)/az%d

        ix=floor(( x-ax%o)/ax%d+1)
        if(ix<   1+1) cycle
        if(ix>ax%n-1) cycle
        xo = ax%o + (ix-1)*ax%d
        fx = ( x-xo)/ax%d

        w00 = (1.-fz) * (1.-fx)
        w01 =     fz  * (1.-fx)
        w10 = (1.-fz) *     fx
        w11 =     fz  *     fx

        data(is) = wfld(iz  ,ix  ) * w00 &
        +          wfld(iz+1,ix  ) * w01 &
        +          wfld(iz  ,ix+1) * w10 &
        +          wfld(iz+1,ix+1) * w11

     end do

     call sep_write(data)
  end do

  deallocate(wfld,data,topo)
  call exit (0)
end program TOPCUT_
