program TopCutSinc_
  !! cut wavefield on topography
  !!
  !! topo = x + i z
  use sep
  use util_

  implicit none
  character(len=128) :: name

  real,    pointer :: wfld(:,:)
  real,    pointer :: data(:)
  complex, pointer :: topo(:)

  type(myaxis) :: ax,az,at,as
  integer      :: ix,iz,it,is
  real         :: xx,zz,tt
  integer      :: jx,jz
  real         :: x, z

  integer      :: method
  real         :: zo,fo

  integer      :: nsz,nsx
  integer      :: isz,isx
  real         :: dx,dz,dr,r,sincr

  write(0,*) "read parameters"
  call sep_init()
  call from_param("method",method,1)

  call hinaxis(az,1)
  call hinaxis(ax,2)
  call hinaxis(at,3)

  name="topo"
  call ainaxis(as,1,name)

  call houaxis(as,1)
  call houaxis(at,2)
  call to_history("n3",1)

  call to_history("esize",4)

  call from_param("nsz",nsz,1)
  call from_param("nsx",nsx,1)

  call sep_close()
  dr = sqrt(az%d**2 + ax%d**2)

  write(0,*) "allocate vectors"
  allocate(wfld(az%n,ax%n))
  allocate(data(as%n))
  allocate(topo(as%n))

  write(0,*) "read topo"
  call sreed("topo",topo,size(topo)*8)


  tloop: do it=1,at%n
     write(0,*) it,at%n
     call sep_read(wfld)
     data=0.

     !     write(0,*) minval(wfld),maxval(wfld)
     sloop: do is=1,as%n   
        z =aimag(topo(is))
        x = real(topo(is))

        iz=floor((z-az%o)/az%d+1)
        if(iz<   1+nsz) cycle
        if(iz>az%n-nsz) cycle

        ix=floor((x-ax%o)/ax%d+1)
        if(ix<   1+nsx) cycle
        if(ix>ax%n-nsx) cycle

        zsinc: do jz=iz-nsz,iz+nsz
           zz=az%o+(jz-1)*az%d
           dz=zz-z

           xsinc: do jx=ix-nsx,ix+nsx
              xx=ax%o+(jx-1)*ax%d
              dx=xx-x

              r=sqrt(dz**2 + dx**2)
              if(abs(r)<epsilon(r)) then
                 sincr = 1.0
              else
                 r = r / dr
                 sincr = sin(r)/r
              end if
              data(is) = data(is) + wfld(jz,jx)*sincr

           end do xsinc
        end do zsinc
     end do sloop
     !     write(0,*) minval(data),maxval(data)        

     call sep_write(data)

  end do tloop

  deallocate(wfld,data,topo)
  call exit (0)
end program TOPCUTSINC_




