program TopMsk_
  !! make topography mask
  !!
  use sep
  use util_

  implicit none
  real,    pointer :: topo(:)
  real,    pointer :: mask(:,:)
  real,    pointer :: sincmap(:,:)

  type(myaxis) :: ax,az
  integer      :: ix,iz
  integer      :: jx,jz
  real         :: xx,zz
  real         :: x, z

  integer      :: method
  real         :: zo,fo

  integer      :: nsz,nsx
  integer      :: isz,isx
  real         :: dx,dz,dr,r,sincr

  call sep_init()
  call from_param("method",method,1)

  call hinaxis(ax,1)
  call from_param("nz",az%n,1)
  call from_param("oz",az%o,0.)
  call from_param("dz",az%d,1.)

  call houaxis(az,1)
  call houaxis(ax,2)

  if(method==2) then
     dr = sqrt(az%d**2 + ax%d**2)

     call from_param("nsz",nsz,1)
     call from_param("nsx",nsx,1)

     allocate(sincmap(2*nsz+1,2*nsx+1))

     do isz=1,2*nsz+1
        dz = (isz-nsz-1)*az%d
        do isx=1,2*nsx+1
           dx = (isx-nsx-1)*az%d

           r=sqrt(dz**2 + dx**2)
           if(abs(r)<epsilon(r)) then
              sincmap(isz,isx) = 1.0
           else
              r = r / dr
              sincmap(isz,isx) = sin(r)/r
           end if

        end do
     end do
  end if

  call to_history("esize",4)
  call sep_close()

  allocate(topo(ax%n))
  allocate(mask(az%n,ax%n))

  call sep_read(topo)

  if(method==2) then

     mask=0.
     do ix=1,ax%n
        x=ax%o+(ix-1)*ax%d

        z=topo(ix)
        iz=floor((z-az%o)/az%d+1)
        iz=maxval((/   1,iz/))
        iz=minval((/az%n,iz/))

        if(iz<   1+nsz) cycle
        if(iz>az%n-nsz) cycle
        if(ix<   1+nsx) cycle
        if(ix>ax%n-nsx) cycle

        mask(iz-nsz:iz+nsz,ix-nsx:ix+nsx) = sincmap
     end do

  elseif(method==1) then
     !! linear
     mask=0.
     do ix=1,ax%n
        zz=topo(ix)
        iz=floor((zz-az%o)/az%d+1)
        iz=maxval((/   1,iz/))
        iz=minval((/az%n,iz/))

        zo = az%o + (iz-1)*az%d
        fo = (zz-zo)/az%d

        mask(iz  ,ix) = (1.-fo)
        mask(iz+1,ix) =     fo
     end do

  else
     !! nearest neighbor
     mask=0.
     do ix=1,ax%n
        zz=topo(ix)
        iz=floor((zz-az%o)/az%d+1)
        iz=maxval((/   1,iz/))
        iz=minval((/az%n,iz/))

        mask(iz  ,ix) = 1
     end do

  end if

  call sep_write(mask)

  call exit (0)
end program TOPMSK_




